@@ -94,7 +94,7 @@ class tag(object):
         # self.name = 'tag'  # not set which means extended tags that don't use create_element will break
         try:
             self.content = ''.join([each.__str__() for each in args])
-            self.attributes = ''.join([''' %s="%s"''' % (key.split('_', 1)[1], value) for key, value in kwargs.items()])
+            self.__attributes__ = ''.join([''' %s="%s"''' % (key.split('_', 1)[1], value) for key, value in kwargs.items()])
         except IndexError as e:
             raise TemplateError(e)
         # except Exception as e:
@@ -110,7 +110,7 @@ class tag(object):
         return
 
     @property
-    def attributes(self):
+    def __attributes__(self):
         try:
             return ''.join([''' %s="%s"''' % (key.split('_', 1)[1], value) for key, value in self.kwargs.items()])
         except IndexError as e:
@@ -118,8 +118,8 @@ class tag(object):
         # except Exception as e:
             # print(e)
 
-    @attributes.setter
-    def attributes(self, ignore):
+    @__attributes__.setter
+    def __attributes__(self, ignore):
         try:
             self.__attributes = ''.join([''' %s="%s"''' % (key.split('_', 1)[1], value) for key, value in self.kwargs.items()])
         except IndexError as e:
@@ -128,7 +128,7 @@ class tag(object):
             # print(e)
 
     def __str__(self):
-        return f"<{self.name}{self.attributes}>{self.content}</{self.name}>"
+        return f"<{self.name}{self.__attributes__}>{self.content}</{self.name}>"
 
     def __mul__(self, other):
         """
@@ -203,7 +203,14 @@ class tag(object):
         return self
 
     def __getitem__(self, index):
-        return self.args[index]
+        # print('getting an item::', index, type(index))
+        if isinstance(index, int):
+            return self.args[index]
+        # elif isinstance(index, str):
+        #     if index.startswith('_'):
+        #         return self.kwargs[index]
+        #     else:
+        #         return getattr(self, index)
 
     def __rshift__(self, item):
         try:
@@ -221,7 +228,7 @@ class tag(object):
     #     except Exception as e:
     #         print(e)
     #         raise ValueError
-    
+
     # def __sub__(self, item):
     #     try:
     #         self.args = self.args - (item,)
@@ -232,9 +239,6 @@ class tag(object):
 
     # def render()
 
-
-
-
     def __getattr__(self, attr):
         """
         allows dot notation for reading attributes
@@ -255,11 +259,15 @@ class tag(object):
         if retry in kwargs:
             return kwargs[retry]
 
-        # raise AttributeError("This attribute or method does not appear to exist on this object:", attr)
+        try:
+            return getattr(super(), attr)
+        except AttributeError:
+            raise AttributeError("This attribute or method does not appear to exist on this object:", attr)
+
         raise AttributeError
 
     # def __repr__(self):
-    #     return f"<{self.name}{self.attributes}>{self.content}</{self.name}>"
+    #     return f"<{self.name}{self.__attributes__}>{self.content}</{self.name}>"
 
     # def __setitem__(self,key,value):
         # self.args[key] = value
@@ -272,7 +280,7 @@ class tag(object):
 
 class closed_tag(tag):
     def __str__(self):
-        return f"<{self.name}{self.attributes} />"
+        return f"<{self.name}{self.__attributes__} />"
 
 
 def tag_init(self, *args, **kwargs):
