@@ -9,6 +9,8 @@ from re import sub
 from itertools import chain, islice
 from collections import Counter
 
+from domonic.decorators import deprecated
+
 
 class Utils(object):
     """ utils """
@@ -247,6 +249,7 @@ class Utils(object):
         from itertools import permutations
         return [''.join(perm) for perm in list(permutations(word))]
 
+    @deprecated
     @staticmethod
     def random_color(self):
         ''' TODO - remove in 0.3 as we have color class. '''
@@ -325,6 +328,29 @@ class Utils(object):
         else:
             return text + "..."
 
+
+    @staticmethod
+    def digits(text=''):
+        """[takes a string of mix of digits and letters and returns a string of digits]
+
+        Args:
+            text (str, optional): [the text to change]. Defaults to ''.
+
+        Returns:
+            [str]: [a string of digits]
+        """
+        if isinstance(text, int):
+            return str(text)
+        elif isinstance(text, float):
+            return str(int(text))
+        elif isinstance(text, str):
+            return ''.join(i for i in text if i.isdigit())
+        else:
+            try:
+                return str(text)
+            except Exception:
+                raise ValueError("text must be a string")
+
     @staticmethod
     def has_internet(url='http://www.google.com/', timeout=5):
         """[check if you have internet connection]
@@ -344,6 +370,24 @@ class Utils(object):
             # print("No internet connection available.")
             return False
 
+    '''
+    @staticmethod
+    def yeahnah(x):
+        """ returns a boolean for any given user reply
+        """
+        reply = x.lower()
+        if reply.lower() in ['yeah', "y", "yes", "yup", "si", "yep", "yeah", "yep"]:
+            return True
+        elif reply.lower() in ['nah', "no", "nope", "n"]:
+            return False
+        else:
+            # return a probability between 0 and 1 for either yes or no based on which list has the most similar words to the input
+            # return max([float(reply.count(x)) / len(reply) for x in ['yeah', "y", "yes", "yup", "si", "yep", "yeah", "yep"]]) > max([float(reply.count(x)) / len(reply) for x in ['nah', "n", "no", "nope", "nop", "nope", "n", "nope"]])
+            # return max([float(reply.count(x)) / len(reply) for x in ['yeah', "y", "yes", "yup", "si", "yep", "yeah", "yep"]]) > 0.5
+        else:
+            return None
+    '''
+
     # def get_ip(self):
     #     """[get the current ip]
 
