@@ -853,12 +853,33 @@ class TestCase(unittest.TestCase):
 
         # selectAll("p").attr("class", "graf").style("color", "red")
         # selectAll("p").attr("class", "test")
-        selectAll("p").append("div")
+        # selectAll("p").append(div)
+        # selectAll("p").append('div')
+        # selectAll("p").attr("class", "graf").style("color", "red")
+        selection = selectAll("p").style("color", "red")
+        # print(page)
+
+        selection.classed("foo bar", True)
+        # print( ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", Math.random() )
+        selection.classed("test", lambda this: Math.random() > 0.2) # seems to always do it
+        # print(page)
+        # print(selection.text())
+        selection.text("CHANGE")
         print(page)
 
         # select("body").append("svg").attr("width", 960).attr("height", 500).append("g").attr("transform", "translate(20,20)").append("rect").attr("width", 920).attr("height", 460)
 
 
+    def test_select2(self):
+
+        # selectAll("p").attr("class", "graf").style("color", "red")
+
+        pass
+
+
+
+
+
 
 if __name__ == '__main__':
     unittest.main()
