@@ -14,7 +14,7 @@ from domonic.svg import *
 from domonic.decorators import silence
 
 
-class domonicTestCase(unittest.TestCase):
+class TestCase(unittest.TestCase):
 
     @silence
     def test_domonic_svg(self):
